"use strict";
var BlockNode = require("../ast").BlockNode;
var ProgramNode = require("../ast").ProgramNode;
var TextNode = require("../ast").TextNode;
var PartialNode = require("../ast").PartialNode;
var appendChild = require("../ast").appendChild;
var usesMorph = require("../ast").usesMorph;
var postprocessProgram = require("../html-parser/helpers").postprocessProgram;
var Chars = require("../html-parser/tokens").Chars;
var forEach = require("../utils").forEach;

var nodeHandlers = {

  program: function(program) {
    var statements = [];
    var node = new ProgramNode(statements, program.strip);
    var i, l = program.statements.length;

    this.elementStack.push(node);

    if (l === 0) { return this.elementStack.pop(); }

    for (i = 0; i < l; i++) {
      this.acceptNode(program.statements[i]);
    }

    this.acceptToken(this.tokenizer.tokenizeEOF());

    postprocessProgram(node);

    // Ensure that that the element stack is balanced properly.
    var poppedNode = this.elementStack.pop();
    if (poppedNode !== node) {
      throw new Error("Unclosed element: " + poppedNode.tag);
    }

    return node;
  },

  block: function(block) {
    switchToHandlebars(this);
    this.acceptToken(block);

    var mustache = block.mustache;
    var program = this.acceptNode(block.program);
    var inverse = block.inverse ? this.acceptNode(block.inverse) : null;
    var strip = block.strip;

    // Normalize inverse's strip
    if (inverse && !inverse.strip.left) {
      inverse.strip.left = false;
    }

    var node = new BlockNode(mustache, program, inverse, strip);
    var parentProgram = this.currentElement();
    appendChild(parentProgram, node);
  },

  content: function(content) {
    var tokens = this.tokenizer.tokenizePart(content.string);

    return forEach(tokens, this.acceptToken, this);
  },

  mustache: function(mustache) {
    switchToHandlebars(this);
    this.acceptToken(mustache);
  },

  comment: function(comment) {
    return;
  },

  partial: function(partial) {
    var node = new PartialNode(partial.partialName.name);
    appendChild(this.currentElement(), node);
    return;
  }
};

function switchToHandlebars(processor) {
  var token = processor.tokenizer.token;

  // TODO: Monkey patch Chars.addChar like attributes
  if (token instanceof Chars) {
    processor.acceptToken(token);
    processor.tokenizer.token = null;
  }
}

exports["default"] = nodeHandlers;